package info.aspirecn.cloud.yysj.filters.error;

import com.alibaba.fastjson.JSON;
import com.netflix.zuul.ZuulFilter;
import com.netflix.zuul.context.RequestContext;
import com.netflix.zuul.exception.ZuulException;
import info.aspirecn.cloud.yysj.model.ResultResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpStatus;
import org.springframework.cloud.netflix.zuul.util.ZuulRuntimeException;
import org.springframework.stereotype.Component;
import org.springframework.util.ReflectionUtils;
import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;

/**
 * 异常处理
 *
 * @author dingliqiang
 */
@Slf4j
@Component
public class ErrorFilter extends ZuulFilter {

    @Override
    public String filterType() {
        return "error";
    }

    @Override
    public int filterOrder() {
        return 0;
    }

    @Override
    public boolean shouldFilter() {
        return true;
    }

    @Override
    public Object run() throws ZuulException {
        PrintWriter writer = null;
        try {
            RequestContext context = RequestContext.getCurrentContext();
            // 获取异常信息
            ZuulException exception = this.findZuulException(context.getThrowable());
            log.error("Zuul 异常处理", exception);
            // 获取响应对象，并设置响应信息
            HttpServletResponse response = context.getResponse();
            response.setContentType("application/json; charset=utf8");
            response.setStatus(HttpStatus.SC_SERVICE_UNAVAILABLE);
            // 设置响应体
            ResultResponse resultResponse = new ResultResponse();
            resultResponse.setCode(500).setMessage("服务器繁忙").setData("");
            String json = JSON.toJSONString(response);
            // 响应信息
            writer = response.getWriter();
            writer.print(json);
        } catch (Exception var5) {
            ReflectionUtils.rethrowRuntimeException(var5);
        } finally {
            if (writer != null) {
                writer.close();
            }
        }
        return null;
    }

    ZuulException findZuulException(Throwable throwable) {
        if (throwable.getCause() instanceof ZuulRuntimeException) {
            return (ZuulException) throwable.getCause().getCause();
        } else if (throwable.getCause() instanceof ZuulException) {
            return (ZuulException) throwable.getCause();
        } else {
            return throwable instanceof ZuulException ? (ZuulException) throwable : new ZuulException(throwable, 500, "系统繁忙");
        }
    }

}
