package info.aspirecn.cloud.yysj.filters.post;

import com.alibaba.fastjson.JSON;
import com.netflix.zuul.ZuulFilter;
import com.netflix.zuul.context.RequestContext;
import com.netflix.zuul.exception.ZuulException;
import info.aspirecn.cloud.yysj.model.ResultResponse;
import info.aspirecn.cloud.yysj.utils.UrlUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StreamUtils;

import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;

/**
 * @author dingliqiang
 */
@Slf4j
@Component
public class StateHandlingFilter extends ZuulFilter {

    @Autowired
    private UrlUtil urlUtil;

    @Override
    public String filterType() {
        return "post";
    }

    @Override
    public int filterOrder() {
        return 0;
    }

    @Override
    public boolean shouldFilter() {
        RequestContext context = RequestContext.getCurrentContext();
        // 获取请求地址
        String url = context.getRequest().getRequestURI();
        // 验证请求地址是否为需要处理状态的地址，是则 true 执行，否则 false 不执行
        return urlUtil.isStateHandingUrl(url);
    }

    @SneakyThrows(value = IOException.class)
    @Override
    public Object run() throws ZuulException {
        RequestContext context = RequestContext.getCurrentContext();
        InputStream stream = context.getResponseDataStream();
        context.setSendZuulResponse(true);
        if (stream != null) {
            // 获取响应体数据
            String body = StreamUtils.copyToString(stream, StandardCharsets.UTF_8);
            // 设置响应内容
            ResultResponse resultResponse = new ResultResponse();
            resultResponse.setMessage("服务器繁忙，请稍后再试");
            context.setResponseStatusCode(HttpStatus.SC_INTERNAL_SERVER_ERROR);
            if (body.length() > 1) {
                resultResponse = JSON.parseObject(body, ResultResponse.class);
            }
            context.setResponseStatusCode(resultResponse.getCode());
            context.getResponse().setContentType("application/json; charset=utf8");
            context.setResponseBody(JSON.toJSONString(resultResponse));

        }
        return null;
    }

}
