package info.aspirecn.cloud.yysj.filters.pre;

import com.alibaba.fastjson.JSON;
import com.netflix.zuul.ZuulFilter;
import com.netflix.zuul.context.RequestContext;
import com.netflix.zuul.exception.ZuulException;
import info.aspirecn.cloud.yysj.model.ResultResponse;
import info.aspirecn.cloud.yysj.model.TokenEntity;
import info.aspirecn.cloud.yysj.service.LoginService;
import info.aspirecn.cloud.yysj.utils.UrlUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author dingliqiang
 */
@Slf4j
@Component
public class AuthenticationFilter extends ZuulFilter {

    @Autowired
    private UrlUtil urlUtil;

    @Autowired
    private LoginService loginService;

    /**
     * pre：可以在请求被路由之前调用
     * route：在路由请求时候被调用
     * post：在route和error过滤器之后被调用
     * error：处理请求时发生错误时被调用
     */
    @Override
    public String filterType() {
        return "pre";
    }

    /**
     * 优先级为0，数字越大，优先级越低
     */
    @Override
    public int filterOrder() {
        return 0;
    }

    @Override
    public boolean shouldFilter() {
        RequestContext context = RequestContext.getCurrentContext();
        // 获取请求地址
        String url = context.getRequest().getRequestURI();
        // 验证请求地址是否为忽略地址
        boolean isExecute = urlUtil.isIngressUrl(url);
        // 如果是忽略地址，则设置为 false 不执行过滤器，否则 true 执行过滤器
        return !isExecute;
    }

    @Override
    public Object run() throws ZuulException {
        RequestContext context = RequestContext.getCurrentContext();
        // 检测 Headers 里面是否有 Token 信息
        context.setSendZuulResponse(false);
        String token = context.getRequest().getHeader("token");
        if (token != null) {
            // 验证 Token 是否正确，并且获取 userId
            TokenEntity tokenEntity = new TokenEntity();
            tokenEntity.setToken(token);
            ResultResponse resultResponse = loginService.checkToken(tokenEntity);
            if (resultResponse == null) {
                // 如果为空，则返回500错误和自定义错误信息
                ResultResponse response = new ResultResponse();
                response.setCode(500).setMessage("服务器繁忙").setData("");
                String json = JSON.toJSONString(response);
                context.getResponse().setContentType("application/json; charset=utf8");
                context.setResponseBody(json);
                context.setResponseStatusCode(500);
            } else if (resultResponse.getCode() != HttpStatus.SC_OK) {
                // 如果响应体 code 不等于 200，则返回响应信息
                context.setResponseStatusCode(resultResponse.getCode());
                context.getResponse().setContentType("application/json; charset=utf8");
                String json = JSON.toJSONString(resultResponse);
                context.setResponseBody(json);
            } else {
                // 默认继续执行后续调用链
                String userId = String.valueOf(resultResponse.getData());
                context.addZuulRequestHeader("userId", userId);
                context.setSendZuulResponse(true);
            }
        }else {
            ResultResponse response = new ResultResponse();
            response.setCode(400).setMessage("请求参数错误，请确认登录后再尝试").setData("");
            String json = JSON.toJSONString(response);
            context.getResponse().setContentType("application/json; charset=utf8");
            context.setResponseBody(json);
            context.setResponseStatusCode(400);
        }
        return null;
    }

}
